<?php
/* ==================================================
 *   Ktai Content Shrinkage
   ================================================== */

define ('KTAI_THUMBNAIL_FILENAME', '.ktai');
define ('KTAI_THUMBNAIL_MAX_SIZE', apply_filters('thumbnail_max_size/ktai_style.php', apply_filters('ktai_thumbnail_max_size', 96)));
define ('KTAI_GD_MEMORY_LIMIT', '256M');
define ('KTAI_SIZE_EXCEED_COLOR', '#808080');
define ('KTAI_SITE_CLASS', 'ktai');
define ('KTAI_PCONLY_SITE_CLASS', 'pconly');
define ('KTAI_REDIR_URL', 'redir.php?_wpnonce=%s&amp;url=%s');
define ('KTAI_DIRECT_LINK_IMAGE_SIZE', apply_filters('direct_link_image_size/ktai_style.php', apply_filters('ktai_direct_link_image_size', 51200)));
define ('KTAI_EXCERPT_LENGTH', 300);
define ('KTAI_MAX_PAGE_NUM', 1000);
define ('KTAI_START_PAGING_REGEX', '@<!--start paging(\[(.*?)\])?-->\n*@');
define ('KTAI_END_PAGING_REGEX', '@\n*<!--end paging(\[(.*?)\])?-->@');

//define ('KTAI_SHRINKAGE_DEBUG', true);

class KtaiShrinkage {
	public  $base;
	private $url;
	private $url_regex;
	private $wpurl;
	private $wpurl_regex;
	private $wpcontent_regex;
	private $upload_path;
	private $short_links;
	private $internal_regex;
	private $short_int_regex;
	private $host_regex;
	private $wp_plugin_url;
	private $scheme_regex;
	private $leave_regex;
	private $mobile_regex;
	private $none_mobile_regex;
	private $num_image;
	private $image_services;
	private $image_services_regex;
	private $skip_shrink_content = false;
	static public $mobile_same_url = array(
		// Use same URL for PC and mobile
		'http://[-\w]+\.blog\d+\.fc2\.com/', 
		'http://(jugem|yaplog)\.jp/[-\w]+/', 
		'http://[-\w]+\.seesaa\.net/', 
		'http://blog\.goo\.ne\.jp/[-\w]+/', 
		'http://blogs\.dion\.ne\.jp/[-\w]+/',
		'http://blog\.auone\.jp/[-\w]+/',
		'http://[-\w]+\.blog\.so-net\.ne\.jp/',
		'http://[-\w]+\.paslog\.jp/',
		'http://[-\w]+\.vox\.com/',
		'http://hb\.afl\.rakuten\.co\.jp/hgc/',
		'http://[-\w]+\.ap\.teacup\.com/',
		'http://pub\.ne\.jp/[-.\w]+/',
		'http://[-\w]+\.blog.shinobi.jp/',
		// Redirect mobile URL from PC
		'http://d\.hatena\.ne\.jp/[-\w]+/',
		'http://blog\.livedoor\.jp/[-\w]+/',
		'http://[-\w]+.(cocolog|air|moe|tea|txt|way)-nifty\.com/',
		'http://[-\w]+\.at\.webry\.info/',
		'http://ameblo\.jp/[-\w]+/',
		'http://[-\w]+\.spaces\.live\.com/',
		'http://plaza\.rakuten\.co\.jp/[-\w]+(/|$)',
		'http://[-\w]+\.blog.drecom.jp/',
	);
	static public $none_mobile_url = array(
		'http://(www|support|app)\.cocolog-nifty\.com/',
	);

// ==================================================
public function __construct() {
	global $Ktai_Style, $wpmu_version;
	$this->base = $Ktai_Style;
	$homeurl = get_bloginfo('url');
	$wpurl = get_bloginfo('wpurl');
	$this->url = $this->base->strip_host($homeurl . '/');
	$this->url_regex = '!^(' . preg_quote($homeurl, '!') . '/|' . preg_quote($this->url, '!') . ')!';
	if (preg_match('!^(https?://[^/]+)!', $homeurl, $host_url)
	&& preg_match('!^(https?://[^/]+)!', $wpurl, $host_wpurl)
	&& $host_url[1] === $host_wpurl[1]) {
		$this->wpurl     = $this->base->strip_host($wpurl . '/');
		$this->wpurl_regex = '!^(' . preg_quote($wpurl, '!') . '/|' . preg_quote($this->wpurl, '!') . ')!';
	} else {
		$this->wpurl     = $wpurl;
		$this->wpurl_regex = '!^' . preg_quote($wpurl, '!') . '!';
	}
	if (! isset($wpmu_version)) {  // single install WordPress
		$this->wpcontent_regex = preg_replace('/\)!$/', ')wp-content/!', $this->wpurl_regex);
		$this->basepath = ABSPATH;
		$this->internal_regex[0] = '!^' . preg_quote($wpurl, '!') . '/?!';
	} else { // WordPress MU
		$this->wpurl .= 'files/';
		$this->wpurl_regex = '!^(' . preg_quote($wpurl, '!') . '/files/|' . preg_quote($this->wpurl, '!') . ')!';
		$this->wpcontent_regex = $this->wpurl_regex;
		$this->basepath = constant('ABSPATH') . constant('UPLOADS');
		$this->internal_regex[0] = '!^' . preg_quote($wpurl, '!') . '/files/?!';
	}
	$this->short_links[0]  = $this->wpurl;
	if (strcmp($this->wpurl, $this->url) !== 0) {
		$this->internal_regex[] = '!^' . preg_quote($homeurl, '!') . '/?!';
		$this->short_links[]  = $this->url;
	}
	$this->short_int_regex = '!^(' . implode('|',  array_map('preg_quote', $this->short_links)) . ')!';
	if (preg_match('!^(https?://[^/]*)/?!', $wpurl, $host)) {
		$this->host_regex = '!^(/|' . preg_quote($host[1], '!') . '/)!';
	} else {
		$this->host_regex = '!^/!';
	}
	
	$this->wp_plugin_url = $this->base->get('plugin_url');
	$this->wp_plugin_url = preg_replace($this->wpurl_regex, '', $this->wp_plugin_url);
	$this->wp_plugin_url = preg_replace('!' . preg_quote($this->base->get('plugin_dir'), '!') . '/?$!', '', $this->wp_plugin_url);
	
	$relative_url = array('^(\./|[-a-zA-Z0-9_.+=%~]+(/|$))');
	$leave_schemes = apply_filters('ktai_leave_scheme', array('tel:', 'tel-av:', 'vtel:', 'mailto:', 'device:', 'location:'));
	$leave_schemes = apply_filters('leave_scheme/ktai_style.php', $leave_schemes);
	$leave_sites = preg_split('/\\s+/', ks_option('ks_treat_as_internal'), -1, PREG_SPLIT_NO_EMPTY);
	$this->scheme_regex = '!^(#|' . implode('|', $leave_schemes) . ')!';
	$this->leave_regex = '!^(#|' . implode('|', array_merge($relative_url,$leave_schemes, array_map('preg_quote', $leave_sites))) . ')!';
	$this->mobile_regex = '!^(' . implode('|', apply_filters('mobile_same_url/ktai_style.php', apply_filters('ktai_mobile_same_url', self::$mobile_same_url))) . ')!';
	$this->none_mobile_regex = '!^(' . implode('|', apply_filters('none_mobile_url/ktai_style.php', apply_filters('ktai_none_mobile_url', self::$none_mobile_url))) . ')!';
	$this->num_image = 0;
	$this->image_services = apply_filters('ktai_image_services', array(
		'^https?://[-\w.]+\.flickr.com/' => array($this, 'get_thumbnail_flickr'),
	));
	$this->image_services = apply_filters('image_services/ktai_style.php', $this->image_services);
	$this->image_services_regex = '!^(' . implode('|', array_keys($this->image_services)) . ')!';

	add_filter('attribute_escape', array($this, 'attribute_escape_filter'), 90, 2);
	add_filter('esc_html', array($this, 'attribute_escape_filter'), 90, 2);
	add_filter('clean_url', array($this, 'clean_url_filter'), 90, 3);
	add_filter('the_title', array($this, 'shrink_title'), 90);
	add_filter('the_content', array($this, 'shrink_content'), 90);
	remove_filter('get_the_excerpt', 'wp_trim_excerpt');
	add_filter('get_the_excerpt',  array($this, 'post_excerpt'), 9);
	add_filter('get_comment_excerpt', array($this, 'comment_excerpt'), 90);
	add_filter('get_comment_text', array($this, 'shrink_content'), 90);
	if ( !defined('KTAI_ADMIN_MODE') || !KTAI_ADMIN_MODE ) {
		add_filter('post_link', array($this->base, 'strip_host'), 90);
		add_filter('page_link', array($this->base, 'strip_host'), 90);
		add_filter('attachment_link', array($this->base, 'strip_host'), 90);
		add_filter('year_link', array($this->base, 'strip_host'), 90);
		add_filter('month_link', array($this->base, 'strip_host'), 90);
		add_filter('day_link', array($this->base, 'strip_host'), 90);
		add_filter('category_link', array($this->base, 'strip_host'), 90);
		add_filter('list_cats', array($this->base, 'strip_host'), 90);
		add_filter('tag_link', array($this->base, 'strip_host'), 90);
	}
	add_filter('edit_post_link', array($this, 'shrink_link'), 10, 2);
	add_filter('edit_comment_link', array($this, 'shrink_link'), 10, 2);
	add_filter('redirect_canonical', array($this, 'complete_url'), 10, 2);
	add_filter('wp_generate_tag_cloud', array($this, 'shrink_tag_cloud'), 90, 3);
	add_filter('wp_dropdown_pages',  array($this, 'shrink_dropdown'), 90, 3);
	add_filter('comment_reply_link', array($this, 'fix_comment_reply_link'), 10, 4);
	add_filter('get_comments_pagenum_link', array($this, 'fix_comments_pagenum_link'));
	if ( $this->base->ktai->get('image_inline_default') != $this->base->ktai->get('image_inline') ) {
		add_filter('ktai_split_page', array($this, 'add_url_inline_image'), 7);
	}
	add_filter('ktai_split_page', array($this, 'split_page'), 9, 2);
	add_filter('ktai_split_page', array($this, 'trim_images'), 20);
	add_filter('img_caption_shortcode', array($this, 'img_caption'), 20, 3);
	return;
}

// ==================================================
public function get($key) {
	return isset($this->$key) ? $this->$key : NULL;
}

// ==================================================
public function added_image() {
	if ($this->skip_shrink_content) {
		return;
	}
	return ++$this->num_image;
}

// ==================================================
public function has_inline_images() {
	return $this->num_image;
}

/* ==================================================
 * @param	string  $safe_text
 * @param	string  $text
 * @return	string  $safe_text
 */
public function attribute_escape_filter($safe_text, $text) {
	return str_replace('&#038;', '&amp;', $safe_text);
}

/* ==================================================
 * @param	string  $url
 * @param	string  $original_url
 * @param	string  $context
 * @return	string  $url
 */
public function clean_url_filter($url, $original_url, $context) {
	if ('display' == $context) {
		$url = str_replace('&#038;', '&amp;', $url);
	}
	return $url;
}

/* ==================================================
 * @param	string  $url
 * @return	string  $url
 */
public function complete_url($redirect_url, $requested_url) {
	if (preg_match('!^://!', $redirect_url)) {
		$request = @parse_url($requested_url);
		$redirect_url = $request['scheme'] . $redirect_url;
	} elseif (preg_match('!^/!', $redirect_url) && preg_match('!^(https?://[^/]*)!', $requested_url, $request)) {
		$redirect_url = $request[1] . $redirect_url;
	}
	return $redirect_url;
}

/* ==================================================
 * @param	string  $title
 * @return	string  $title
 */
public function shrink_title($title) {
	$phrase[0]     = str_replace('%s', '', __('Protected: %s'));
	$phrase_regex[0] = '/^' . str_replace('%s', '(.*)', preg_quote(__('Protected: %s'), '/')) . '$/';
	$icon[0]       = str_replace('%s', '$1', __('Protected: %s'));
	$icon[0]       = str_replace($phrase[0], '<img localsrc="501" alt="' . $phrase[0] . '" />', $icon[0]);
	$phrase[1]     = str_replace('%s', '', __('Private: %s'));
	$phrase_regex[1] = '/^' . str_replace('%s', '(.*)', preg_quote(__('Private: %s'), '/')) . '$/';
	$icon[1]       = str_replace('%s', '$1', __('Private: %s'));
	$icon[1]       = str_replace($phrase[1], '<img localsrc="279" alt="' . $phrase[1] . '" />', $icon[1]);
	return preg_replace($phrase_regex, $icon, $title);
}

/* ==================================================
 * @param	string  $link
 * @return	string  $link
 * @since	2.0.0
 */
public function shrink_link($link, $id) {
	return preg_replace('/ (class|title)=([\'"])([^\\\\]*?(\\\\.[^\\2\\\\]*?)*)\\2/s', '', $link);
}

/* ==================================================
 * @param	string  $content
 * @return	string  $content
 */
public function convert_links($content) {
	for ($offset = 0, $replace = 'X' ; 
	     preg_match('!<a ([^>]*?)>(.*?)</a>!s', $content, $l, PREG_OFFSET_CAPTURE, $offset) ; 
	     $offset += strlen($replace))
	{
		$orig      = $l[0][0];
		$offset    = $l[0][1];
		$attr      = $l[1][0];
		$label     = $l[2][0];
		$link_html = $label; // default is stripping links
		$replace   = $orig;
		if (! preg_match('/href=([\'"])([^\\\\]*?(\\\\.[^\\1\\\\]*?)*)\\1/s', $attr, $h)) {
			continue;
		}
		$q    = $h[1];
		$href = $h[2];
		preg_match('/class=([\'"])([^\\\\]*?(\\\\.[^\\1\\\\]*?)*)\\1/s', $attr, $class);
		if (preg_match('!<img [^>]*?src=([\'"])([^\\\\]*?(\\\\.[^\\1\\\\]*?)*)\\1[^>]*? ?/?>!s', $label, $image)) {
			$src = $image[2];
			if (preg_match('!' . preg_quote($this->wp_plugin_url, '!') . '!', $src)) { // skip plug-in's icon
				continue; // leave links
			} elseif (ks_is_image_inline() && preg_match($this->image_services_regex, $href) && preg_match($this->image_services_regex, $src)) {
				foreach ($this->image_services as $pat => $callback) {
					if (preg_match('!' . $pat . '!', $src)) {
						$href = call_user_func($callback, $src, $href);
						break;
					}
				}
				$link_html = str_replace($h[0], 'href=' . $q . $href . $q, $orig); // leave image services URL (Flickr etc)
			} elseif (preg_match($this->wpurl_regex, $href) && preg_match($this->wpurl_regex, $src)) { // both internal link
				$path = preg_replace($this->wpurl_regex, $this->basepath, $href);
				if (! is_dir($path) && $imagesize = @filesize($path)) { // a thumbnail linked to original image
					if (ks_is_image_inline() && $imagesize && $imagesize <= min($this->base->get('cache_size'), KTAI_DIRECT_LINK_IMAGE_SIZE)) {
						$link_html = $orig; // link to image
					} else {
						$thumbnail = str_replace('<img ', '<img has_orig="true" ', $label); // inform existance of original to images_to_link()
						$link_html = $thumbnail . '<img src="' . $href . '" alt="' . sprintf(__('Original(%dKB)', 'ktai_style'), intval($imagesize / 1024)) . '" filesize="' . $imagesize . '"/>'; // pass filesize to images_to_link()
					}
				} else { // internal link to other than images
					$link_html = $label . '(<a href="' . $href . '">' . __('Link Target', 'ktai_style') . '</a>)';
				}
			} elseif (preg_match($this->host_regex, $href) && preg_match($this->host_regex, $src) && $_SERVER['DOCUMENT_ROOT']) { // both in-host link
				$path = preg_replace($this->host_regex, $_SERVER['DOCUMENT_ROOT'] . '/', $href);
				if (! is_dir($path) && $imagesize = @filesize($path)) { // a thumbnail linked to original image
					$thumbnail = str_replace('<img ', '<img has_orig="true" ', $label); // inform existance of original to images_to_link()
					$link_html = $thumbnail . '<img src="' . $href . '" alt="' . sprintf(__('Original(%dKB)', 'ktai_style'), intval($imagesize / 1024)) . '" filesize="' . $imagesize . '"/>'; // pass filesize to images_to_link()
				} else { // internal link to other than images
					$link_html = $label . '(<a href="' . $href . '">' . __('Link Target', 'ktai_style') . '</a>)';
				}
			} else { // external link
				$link_html = $this->rewrite_link($href, __('Link Target', 'ktai_style'), $class);
				if (is_null($link_html)) {
					$link_html = '<a href="' . $href . '">' . __('Link Target', 'ktai_style') . '</a>';
				}
				$link_html = "{$label}($link_html)";
			}
		} else {
			$link_html = $this->rewrite_link($href, $label, $class);
		}
		$replace = apply_filters('ktai_convert_links', $link_html, $orig, $href, $label);
		$replace = apply_filters('convert_links/ktai_style.php', $replace, $orig, $href, $label);
		if (! is_null($replace)) {
			$content = substr_replace($content, $replace, $offset, strlen($orig)); // convert links
		} else {
			$offset += strlen($orig);
		}
	}
	return $content;
}

/* ==================================================
 * @param   string  $href
 * @param	string  $label
 * @param	string  $class
 * @return	string  $link_html
 */
public function rewrite_link($href, $label, $class) {
	$clipped = preg_replace($this->internal_regex, $this->short_links, $href, 1, $is_internal);
	$leave_regex = ks_is_loggedin() ? $this->scheme_regex : $this->leave_regex;
	if ($is_internal) {
		$link_html = '<a href="' . $clipped . '">' . $label . '</a>';
	} elseif (preg_match($this->short_int_regex, $href) 
	       || preg_match($leave_regex, $href)) {
		$link_html = NULL; // leave links
	} else {
		$icon = '<img localsrc="70" alt="' . __('[external]', 'ktai_style') . '" />';
		$nonce = $this->base->create_anon_nonce('redir_' . md5($href) . md5($_SERVER['HTTP_USER_AGENT']));
		if (preg_match('/(^| )' . preg_quote(KTAI_PCONLY_SITE_CLASS, '/') . '( |$)/', $class[2]) || $this->none_mobile_sites($href)) {
			$pconly_html = '&amp;' . KTAI_PCONLY_SITE_CLASS . '=true';
		} else {
			$pconly_html = '';
		}
		$redir_link = '<a href="' . ks_plugin_url(KTAI_NOT_ECHO) . sprintf(KTAI_REDIR_URL, attribute_escape($nonce) . $pconly_html, rawurlencode($href)) . '">';
		$direct_link = '<a href="' . attribute_escape($href) . '">';
		$colored_label = '<font color="' . ks_option('ks_external_link_color') . '">' . $label . '</font></a>';
		if (ks_is_loggedin()) {
			$link_html = $icon . $redir_link . $colored_label;
		} elseif (preg_match('/(^| )' . preg_quote(KTAI_SITE_CLASS, '/') . '( |$)/', $class[2]) || $this->has_mobile_sites($href)) {
			$link_html = $direct_link . $label . '</a>';
		} elseif ($this->base->get('use_redir')) {
			$link_html = $icon . $redir_link . $colored_label;
		} else {
			$link_html = $direct_link . $colored_label;
		}
		$link_html = apply_filters('ktai_external_link', $link_html, $href, $label, $icon);
		$link_html = apply_filters('external_link/ktai_style.php', $link_html, $href, $label, $icon);
	}
	return $link_html;
}

// ==================================================
private function has_mobile_sites($url) {
	if (preg_match($this->none_mobile_regex, $url)) {
		return false;
	}
	return preg_match($this->mobile_regex, $url);
}

// ==================================================
private function none_mobile_sites($url) {
	return preg_match($this->none_mobile_regex, $url);
}

/* ==================================================
 * @param	string  $content
 * @return	string  $content
 */
public function convert_images($content) {
	for ($offset = 0, $replace = 'X'; 
	     preg_match('!<img ([^>]*?)src=([\'"])([^\\\\]*?(\\\\.[^\\2\\\\]*?)*)\\2([^>]*?) ?/?>!s', $content, $img, PREG_OFFSET_CAPTURE, $offset) ; 
	     $offset += strlen($replace))
	{
		$orig    = $img[0][0];
		$offset  = $img[0][1];
		$q       = $img[2][0];
		$src     = $img[3][0];
		$attr    = $img[1][0] . $img[5][0];
		$replace = $orig;
		if (preg_match('/local$/', $img[1][0])) { // ezweb pict chars
			continue;
		}
		list($replace, $has_image) = $this->image_to_link($orig, $src, $q, $attr);
		$replace = apply_filters('ktai_image_to_link', $replace, $orig, $src);
		$replace = apply_filters('image_to_link/ktai_style.php', $replace, $orig, $src);
		if (! is_null($replace)) {
			$content = substr_replace($content, $replace, $offset, strlen($orig));
			if ($has_image) {
				$this->added_image();
			}
		} else {
			$offset += strlen($orig);
		}
	}
	return $content;
}

/* ==================================================
 * @param	string  $html
 * @param	string  $src
 * @param	string  $q
 * @param	string  $attr_str
 * @return	string  $replace
 * @return	boolean $has_image
 */
private function image_to_link($html, $src, $q, $attr_str) {
	$replace = $html;
	$has_image = false;
	$attr = array();
	if (preg_match_all('/\b(alt|title|class|style|align)=([\'"])([^\\\\]*?(\\\\.[^\\2\\\\]*?)*)\\2/s', $attr_str, $attr_list, PREG_SET_ORDER)) {
		foreach($attr_list as $a) {
			$attr[$a[1]] = $a[3];
		}
	}
	if (isset($attr['alt'])) {
		$alt = stripslashes($attr['alt']);
		if (empty($alt) && (! isset($attr['class']) || ! preg_match('/(^| )(wp-image-\d+)( |$)/', $attr['class']))) {
			$replace = ''; // hide images if the alt string is empty. 
			return array($replace, $has_image);
		}
	} else {
		$alt = NULL;
	}
	if (empty($alt)) {
		if (isset($attr['title'])) {
			$alt = $attr['title'];
		} else {
			$url = parse_url($src);
			$alt = basename($url['path']);
		}
	}
	if (preg_match('/(width|height)=(([\'"])1\\3|1\b)/s', $attr_str)) {
		$replace = ''; // hide 1 pixel width/height images
	} elseif (isset($attr['style']) && preg_match('/\b(visibility:\s*hidden|disypay:\s*none)/', $attr['style'])) {
		$replace = ''; // hide hidden styled images
	} elseif (isset($attr['class']) && preg_match('/(^| )(wp-smiley|' . preg_quote(KTAI_SITE_CLASS) . ')( |$)/', $attr['class'])) {
		$short_src = preg_replace($this->host_regex, '/', $src);
		$replace = '<img src=' . $q . $short_src . $q . ' alt="' . $alt . '" class="' . $attr['class'] . '"';
		if (isset($attr['style'])) {
			$replace .= ' style="' . $attr['style'] . '"';
		}
		if (isset($attr['align'])) {
			$replace .= ' align="' . $attr['align'] . '"';
		}
		$replace .= ' />';
	} elseif (preg_match('!' . preg_quote($this->wp_plugin_url, '!') . '!', $src)) { // plug-in's icon or what
		if ($this->base->get('show_plugin_icon')) {
			$has_image = true;
			$replace = ks_is_image_inline() ? preg_replace('!(src=[\'"])https?://[^/]*!', '$1', $html) : "[$alt]"; // strip host
		} else {
			$replace = "[$alt]";
		}
	} elseif (preg_match('/filesize="(\d*)"/', $attr_str, $filesize)) { // original image for thumbnail passed by convert_links()
		$cache_size = $this->base->get('cache_size');
		if ($filesize[1] && $cache_size > 0 && $filesize[1] > $cache_size) {
			$replace = '<font color="' . KTAI_SIZE_EXCEED_COLOR . '">' . $alt . '</font>]';
		} else {
			$replace = '<a href=' . $q . $src . $q . '>' . $alt . '</a>]';
		}

	} else {
/*		$upload_path = get_option('upload_path');
		$upload_url = get_option('upload_url_path');
		$upload_url_regex = '!^' . preg_quote($upload_url, '!') . '!';
		if (file_exists($upload_path) && $upload_url && preg_match($upload_url_regx, $src)) {
			list($thumb_path, $thumb_url, $img_path) = $this->check_thumbnail($src, $upload_url_regex, $upload_path, $upload_url);
		}
*/
		if (preg_match($this->wpurl_regex, $src)) {
			list($thumb_path, $thumb_url, $img_path) = $this->check_thumbnail($src, $this->wpurl_regex, $this->basepath, $this->wpurl);
		} elseif (preg_match($this->host_regex, $src) && $_SERVER['DOCUMENT_ROOT']) {
			list($thumb_path, $thumb_url, $img_path) = $this->check_thumbnail($src, $this->host_regex, $_SERVER['DOCUMENT_ROOT'] . '/', '/');
		} else {
			list($thumb_path, $thumb_url, $img_path) = array(NULL, NULL, NULL);
			foreach ($this->image_services as $pat => $callback) {
				if (preg_match('!' . $pat . '!', $src)) {
					$thumb_url = $src = call_user_func($callback, $src);
					$has_image = true;
					break;
				}
			}
		}
		if ( defined('KTAI_SHRINKAGE_DEBUG') && KTAI_SHRINKAGE_DEBUG && is_ks_error($thumb_path)) {
			$replace = '[[' . $thumb_path->getMessage() . ']]';
		} elseif (ks_is_image_inline() && $thumb_url) {
			$replace = '<img src="' . $thumb_url . '" alt="' . $alt . '"';
			if (isset($attr['style'])) {
				$replace .= ' style="' . $attr['style'] . '"';
			}
			if (isset($attr['align'])) {
				$replace .= ' align="' . $attr['align'] . '"';
			}
			$replace .= preg_match('/has_orig="true"/', $attr_str) ? ' />[' : ' />';
			$has_image = true;
		} else {
			$replace = '[<img localsrc="94" alt="' . __('IMAGE:', 'ktai_style') . '" />';
			if (! $img_path) { // link to a image of external sites
				$replace .= '<a href=' . $q . $src . $q . '>' . $alt . '</a>';
			} elseif (! is_dir($img_path) && $size = @filesize($img_path) && $size <= $this->base->get('cache_size')) { // link to the image
				$replace .= '<a href=' . $q . $src . $q . '>' . $alt . '</a>';
				$has_image = ! is_ks_error($thumb_path);
			} elseif ($thumb_path && $thumb_url) { // link to a thumbnail
				$replace .= '<a href=' . $q . $thumb_url . $q . '>' . $alt . '</a>';
				$has_image = ! is_ks_error($thumb_path);
			} else { // no link to images
				$replace .= '<font color="' . KTAI_SIZE_EXCEED_COLOR . '">' . $alt . '</font>';
			}
			$replace .= preg_match('/has_orig="true"/', $attr_str) ? ' | ' : ']';
		}
	}
	if (isset($attr['class']) && preg_match('/\b(align(left|right))\b/', $attr['class'], $align)) {
		$replace = ks_image_alignment($replace, $align[1]);
	}
	return array($replace, $has_image);
}

/* ==================================================
 * @param	string  $img_src
 * @param	string  $url_regex
 * @param	string  $path_prefix
 * @param	string  $short_url
 * @return	string  $thumb_path
 * @return	string  $thum_burl
 * @return	string  $img_path
 */
private function check_thumbnail($img_src, $url_regex, $path_prefix, $short_url) {
	$img_path = preg_replace($url_regex, $path_prefix, $img_src);
	$thumb_path = $this->get_thumbnail($img_path);
	if (is_ks_error($thumb_path)) {
		return array($thumb_path, NULL, $img_path);
	}
	$thumb_url = preg_replace('!^' . preg_quote($path_prefix, '!') . '!', $short_url, $thumb_path);
	return array($thumb_path, $thumb_url, $img_path);
}

/* ==================================================
 * @param	string  $path
 * @return	string  $thumb
 */
public function get_thumbnail($path) {
	if (! preg_match('!^(cropped-)?(.*?)(\.thumbnail|-\d+x\d+)?(\.[^.]+)?$!', basename($path), $file)) {
		return false;
	}
	$base = $file[2];
	$ext = isset($file[4]) ? $file[4] : '';
	$orig = dirname($path) . '/' . $base . $ext;
	if (isset($file[3]) && file_exists($orig)) {		
		$target = $orig; // Use the original image to make a smaller thumbnail.
		$result = $this->create_alter_image($path, false);
		if ( defined('KTAI_SHRINKAGE_DEBUG') && KTAI_SHRINKAGE_DEBUG && is_ks_error($result)) {
			return $result;
		}
	} else {
		$target = $path;
	}
	$thumb = dirname($path) . '/' . $base . KTAI_THUMBNAIL_FILENAME . $ext;
	if (! file_exists($thumb)) {
		$thumb = $this->create_thumbnail($target, $thumb);
	}
	return $thumb;
}

/* ==================================================
 * @param	string  $img_path
 * @param	string  $thumb_path
 * @return	mix     $result
 */
private function create_thumbnail($img_path, $thumb_path) {
	$size = $this->create_alter_image($img_path, true);
	if (is_ks_error($size)) {
		return $size;
	}
	try {
		$width  = $size[0];
		$height = $size[1];
		$type   = $size[2];
		$image  = $size['image'];
		if ($width <= KTAI_THUMBNAIL_MAX_SIZE && $height <= KTAI_THUMBNAIL_MAX_SIZE) { // No need to make thumbnail
			return $img_path;
		}
		if ($width > $height) {
			$image_ratio = $width / KTAI_THUMBNAIL_MAX_SIZE;
			$new_width  = KTAI_THUMBNAIL_MAX_SIZE;
			$new_height = $height / $image_ratio;
		} else {
			$image_ratio = $height / KTAI_THUMBNAIL_MAX_SIZE;
			$new_height = KTAI_THUMBNAIL_MAX_SIZE;
			$new_width = $width / $image_ratio;
		}
		$thumbnail = @imagecreatetruecolor($new_width, $new_height);
		if (! $thumbnail) {
			throw new KtaiError('Cannot initialize for thumbnail');
		}
		if (function_exists('imageantialias')) {
			imageantialias($thumbnail, true);
		}
		if (! imagecopyresampled($thumbnail, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height)) {
			throw new KtaiError('Resample failed');
		}
	
		// move the thumbnail to it's final destination
		$other_path = NULL;
		$result_other = NULL;
		switch ($type) {
		case IMAGETYPE_GIF:
			$result = @imagegif($thumbnail, $thumb_path);
			if ($result) {
				$other_path = preg_replace('|\.gif$|i', '.png', $thumb_path);
				$result_other = @imagepng($thumbnail, $other_path);
			}
			break;
		case IMAGETYPE_PNG:
			$result = @imagepng($thumbnail, $thumb_path);
			if ($result) {
				$other_path = preg_replace('|\.png$|i', '.gif', $thumb_path);
				$result_other = @imagegif($thumbnail, $other_path);
			}
			break;
		case IMAGETYPE_JPEG:
		default:
			$result = @imagejpeg($thumbnail, $thumb_path);
		break;
		}
		imagedestroy($thumbnail);
		if (! $result || ! file_exists($thumb_path)) {
			throw new KtaiError('Thumbnail file not written');
		}
		chmod($thumb_path, 0646);
		if ($other_path && $result_other) {
			chmod($other_path, 0646);
		}
		return $thumb_path;
	
	} catch (KtaiError $e) {
		return $e;
	}
}

/* ==================================================
 * @param	string   $img_path
 * @param	boolean  $return_image
 * @return	resource $image
 */
private function create_alter_image($img_path, $return_image = true) {
	try {
		if (! function_exists('imagecreatetruecolor')) {
			throw new KtaiError('GD not available');
		}
		if (empty($img_path)) {
			throw new KtaiError('No file name');
		}
		if (! file_exists($img_path)) {
			throw new KtaiError('No such a file:' . $img_path);
		}		
		$size = getimagesize($img_path);
		if (! $size) {
			throw new KtaiError('Cannot access to image:' . $img_path);
		}
		$width  = $size[0];
		$height = $size[1];
		$type   = $size[2];
		if ($width <= 0 || $height <= 0) {
			throw new KtaiError('Zero size image');
		}
		$other_path = NULL;
		$result = NULL;
		@ini_set('memory_limit', KTAI_GD_MEMORY_LIMIT); // Use memory to load the image
		switch ($type) {
		case IMAGETYPE_GIF:
			$image = imagecreatefromgif($img_path);
			$other_path = preg_replace('|\.gif$|i', '.png', $img_path);
			if ($image && ! file_exists($other_path)) {
				$result = @imagepng($image, $other_path);
			}
			break;
		case IMAGETYPE_PNG:
			$image = imagecreatefrompng($img_path);
			$other_path = preg_replace('|\.png$|i', '.gif', $img_path);
			if ($image && ! file_exists($other_path)) {
				$result = @imagegif($image, $other_path);
			}
			break;
		case IMAGETYPE_JPEG:
			if ($return_image) {
				$image = imagecreatefromjpeg($img_path);
			}
			break;
		default:
			throw new KtaiError(sprintf('Can\'t handle image type "%1$s" of file: %2$s', $size['mime'], $img_path));
			break;
		}
		if ($return_image && ! $image) {
			throw new KtaiError('Invalid image file: ' . $img_path);
		}
		if ($other_path && $result) {
			chmod($other_path, 0646);
		}
		if ($result === false) {
			throw new KtaiError('Cannot write PNG/GIF image: ' . $other_path);
		}
		if ($return_image) {
			$size['image'] = $image;
			return $size;
		} else {
			return $result;
		}

	} catch (KtaiError $e) {
		return $e;
	}
}

/* ==================================================
 * @param	string  $img_src
 * @param	string  $img_href
 * @return	string  $thumb_url
 */
public function get_thumbnail_flickr($img_src, $img_href = '') {
	if ($img_href) {
		$thumb_url = preg_replace('!(/\d+_[0-9a-z]+)(_[stmb])?(\.jpg)$!', '$1_m$3', $img_src);
	} else {
		$thumb_url = preg_replace('!(/\d+_[0-9a-z]+)(_[stmb])?(\.jpg)$!', '$1_t$3', $img_src);
	}
	return $thumb_url;
}

/* ==================================================
 * @param	string  $content
 * @return	string  $content
 */
public function strip_styles_scripts_del($content) {
	$content = preg_replace('#<(script|style)[^>]*>.*?</\\1>#s', '', $content);
	$content = preg_replace('#<!--[^[].*?-->\\s*#s', '', $content);
	$content = preg_replace('#<del[^>]*>.*?</del>\\s*#s', '', $content);
	return $content;
}

/* ==================================================
 * @param	string  $content
 * @return	string  $content
 */
public function shrink_content($content) {
	if (! $this->skip_shrink_content) {
		$content = $this->strip_styles_scripts_del($content);
		$content = $this->base->filter_tags($content);
		$content = $this->convert_links($content);
		$content = $this->convert_images($content);
	}
	return $content;
}

/* ==================================================
 * @param	string  $text
 * @param	string  $text
 */
public function trim_excerpt($text) {
	$text = $this->base->filter_tags($text);
	$text = $this->strip_styles_scripts_del($text);
	$text = str_replace(']]>', ']]&gt;', $text);
	$text = strip_tags($text, '<img>');
	$text = preg_replace('!<img ([^>]+?)(localsrc="\w+")?([^>]+?)>!e', '"$2" ? "<img $2 />" : ""', $text);
	$len = apply_filters('excerpt_length', KTAI_EXCERPT_LENGTH);
	if (strlen($text) > $len) {
		$text = ks_cut_html($text, $len, 0) . apply_filters('excerpt_more', '...');
	}
	return $text;
}

/* ==================================================
 * @param	string  $text
 * @param	string  $text
 * Based on wp_trim_excerpt at formatting.php of WP 2.5
 */
public function post_excerpt($text) { // Fakes an excerpt if needed
	if ( '' == $text ) {
		$text = get_the_content('');
		if (function_exists('strip_shortcodes')) {
			$text = strip_shortcodes($text);
		}
		$this->skip_shrink_content = true;
		$text = apply_filters('the_content', $text);
		$this->skip_shrink_content = false;
		$text = $this->trim_excerpt($text);
	}
	return $text;
}

/* ==================================================
 * @param	string  $excerpt
 * @return	string  $excerpt
 */
public function comment_excerpt($excerpt) {
	global $comment;
	return $this->trim_excerpt($comment->comment_content);
}

/* ==================================================
 * @param	string  $buffer
 * @param	int     $page_num
 * @return	string  $paged_content
 */
public function split_page($buffer, $page_num) {
	if ($page_num > KTAI_MAX_PAGE_NUM) {
		$page_num = KTAI_MAX_PAGE_NUM;
	} elseif ($page_num < 1) {
		$page_num = 1;
	}
	list($header, $buffer, $footer, $sep) = $this->separate_buffer($buffer);

	$buffer = preg_replace('/>[ \t]*\n+[ \t]*</', '><', $buffer);
	if (preg_match('/<input type="hidden" name="post_password" value="(.*?)"/s', $buffer, $match)) {
		$post_password = $match[1];
	} else {
		$post_password = '';
	}
	list($navi, $del_accesskey) = $this->get_split_page_navi(KTAI_MAX_PAGE_NUM +1 ,true, $post_password);
	$page_size = $this->base->get('page_size') - strlen($header . $navi . '<hr /><hr />' . $navi . $footer) - 32; // 32-byte is space for adding tags by force_balance_tags()
	if ($page_size < 256) { // too small
		if (preg_match('@<body[^>]*>@s', $header, $s, PREG_OFFSET_CAPTURE)) {
			$move2body = substr_replace($header, '', 0, $s[0][1] + strlen($s[0][0]));
			$header = substr_replace($header, '', $s[0][1] + strlen($s[0][0]));
		} else {
			$move2body = $header;
			$header = '';
		}
		$buffer = $move2body . $buffer . $footer;
		$footer = '';
		$page_size = $this->base->get('page_size') - strlen($header . $navi . '<hr /><hr />' . $navi . $footer) - 32;
	}

	$start_tags = '';
	$terminator = '<!--KTAI_TERMINATOR_' . md5(uniqid()) . '-->';
	$marker = 0;
	$buffer_length = strlen($buffer);
	for ($count = 0 ; $count < $page_num ; $count++) {
		$fragment = ks_cut_html($buffer, $page_size, $marker, $this->base->get('charset'));
		$fragment = preg_replace('/\x1b\$[GEFOPQ]?$/', '', $fragment);
		if (preg_match('/(\x1b\$[GEFOPQ])[!-z]+$/', $fragment, $pict_sequence)) {
			$complemention = "\x0f"; // complete softbank pictgram shift-in
		} else {
			$complemention = '';
		}
		$quoted = str_replace(array("<\x0f", ">\x0f"), array("&lt;\x0f", "&gt;\x0f"), $start_tags . $fragment . $terminator); // protect softbank pictograms
		$balanced = force_balance_tags($quoted);
		preg_match("/$terminator(.*)\$/", $balanced, $added_html);
		$complemented = preg_replace('/\x1b\$[GEFOPQ]?\x1b/', '', $start_tags . $fragment . $complemention . @$added_html[1]);
		if (preg_match_all('!</([^<>]*)>!', @$added_html[1], $added_tags)) {
			$start_tags = '<' . implode('><', array_reverse($added_tags[1])) . '>'; // store complemented tags to next fragment
			if (strpos($start_tags, '<ol>') !== false) {
				$start_tags = $this->detect_nesting_list($balanced, $start_tags);
			}
			$start_tags .= (isset($pict_sequence[1]) ? $pict_sequence[1] : '');
		} else {
			$start_tags = (isset($pict_sequence[1]) ? $pict_sequence[1] : '');
		}
		$marker += strlen($fragment);
		if ($marker >= $buffer_length) {
			$count++;
			break;
		}
	}

	if (strlen($fragment) < $buffer_length && isset($added_html[1])) {
		list($navi, $del_accesskey) = $this->get_split_page_navi($count, ($marker +1 < $buffer_length), $post_password);
		if ($del_accesskey) { // delete redundant access keys
			$complemented = preg_replace('/(<(a|label)[^>]*?) accesskey="[' . $del_accesskey . ']"([^>]*?)>/', '$1$3>', $complemented);
		}
		return $header . 
			$sep['start']['before'] . $navi . $sep['start']['after'] . 
			$complemented . 
			$sep['end']['before']   . $navi . $sep['end']['after'] . 
			$footer;
	} else {
		return $header . $buffer . $footer;
	}
}

/* ==================================================
 * @param	string  $buffer
 * @return	string  $buffer
 */
private function separate_buffer($buffer) {
	$sep = array(
		'start' => array('before' => '', 'after' => '<hr />'),
		'end'   => array('before' => '<hr />', 'after' => ''),
	);
	if (preg_match(KTAI_START_PAGING_REGEX, $buffer, $s, PREG_OFFSET_CAPTURE)) {
		$header = substr_replace($buffer, '', $s[0][1]);
		$buffer = substr_replace($buffer, '', 0, $s[0][1] + strlen($s[0][0]));
		if (isset($s[1])) {
			list($before, $after) = explode(',', $s[2][0]);
			$sep['start']['before'] = strpos($s[2][0], ',') !== false ? $before : '';
			$sep['start']['after']  = strpos($s[2][0], ',') !== false ? $after : $before;
		}
	} elseif (preg_match('@<body[^>]*>@s', $buffer, $s, PREG_OFFSET_CAPTURE)) {
		$header = substr_replace($buffer, '', $s[0][1] + strlen($s[0][0]));
		$buffer = substr_replace($buffer, '', 0, $s[0][1] + strlen($s[0][0]));
	} else {
		$header = '';
	}
	if ($num_match = preg_match_all(KTAI_END_PAGING_REGEX, $buffer, $s, PREG_OFFSET_CAPTURE | PREG_SET_ORDER)) {
		$s = $s[$num_match -1];
		$footer = substr_replace($buffer, '', 0, $s[0][1] + strlen($s[0][0]));
		$buffer = substr_replace($buffer, '', $s[0][1]);
		if (isset($s[1])) {
			list($before, $after) = explode(',', $s[2][0]);
			$sep['end']['before'] = $before;
			$sep['end']['after']  = $after;
		}
	} else {
		if (preg_match('!<hr [^/>]*/>\s*(<div>\s*)?<a (id|name)="tail"!s', $buffer, $s, PREG_OFFSET_CAPTURE)) {
			$footer = substr_replace($buffer, '', 0, $s[0][1]);
			$buffer = substr_replace($buffer, '', $s[0][1]);
		} else {
			list($buffer, $footer) = preg_split('@</body@', $buffer, 2);
			$footer = '</body' . $footer;
		}
	}

	return array($header, $buffer, $footer, $sep);
}

/* ==================================================
 * @param	int     $num
 * @param	boolean $rest
 * @param	string  $post_password
 * @return	string  $navi
 * @return	int     $page_num
 */
private function get_split_page_navi($num, $rest, $post_password) {
	$link = htmlspecialchars(remove_query_arg('kp', $_SERVER['REQUEST_URI']), ENT_QUOTES);
	$link .= (strpos($link, '?') === false) ? '?kp=': '&amp;kp=';
	$del_accesskey = '';
	if ($num == 2) {
		$prev = _ks_internal_link(preg_replace('/(\?|&(amp;)?)kp=/', '', $link), '*', __('*.Prev', 'ktai_style'), $post_password) . ' | ';
		$del_accesskey .= '*';
	} elseif ($num >= 3) {
		$prev = _ks_internal_link($link . intval($num -1), '*', __('*.Prev', 'ktai_style'), $post_password) . ' | ';
		$del_accesskey .= '*';
	} else {
		$prev = '';
	}
	if ($rest) {
		$next = ' | ' . _ks_internal_link($link . intval($num +1), '#', __('#.Next', 'ktai_style'), $post_password);
		$del_accesskey .= '#';
	} else {
		$next = '';
	}
	$navi = sprintf(__('<div align="center">Splitting the page for mobile: %1$s page #%2$d %3$s</div>', 'ktai_style'), $prev, $num, $next);
	$navi = apply_filters('ktai_split_page_navi', $navi, $prev, $num, $next);
	$navi = apply_filters('split_page_navi/ktai_style.php', $navi, $prev, $num, $next);
	if (function_exists('mb_convert_encoding')) {
		$navi = mb_convert_encoding($navi, $this->base->get('charset'), get_bloginfo('charset'));
	}
	return array($navi, $del_accesskey);
}

/* ==================================================
 * @param	string  $balanced
 * @param	string  $start_tags
 * @return	string  $start_tags
 */
private function detect_nesting_list($balanced, $start_tags) {
	preg_match_all('!</?[ou]l([^>]*)>!s', $balanced, $lists, PREG_OFFSET_CAPTURE);
	$open[0] = '<ol>';
	do {
		$close = array_pop($lists[0]);
	} while (strpos($close, '</ol>') !== false);
	$max_ol_level = preg_match_all('/<ol>/', $start_tags, $ol);
	for ($ol_level = 0 ; $ol_level < $max_ol_level ; $ol_level++) {
		$inside[$ol_level][0] = array('start' => strlen($close[0]), 'end' => $close[1]);
	}
	$level = 0;
	$ol_level = 0;
	$below_level[$ol_level] = $level +1;
	$entered[$ol_level] = 1; // to make sure
	foreach (array_reverse($lists[0]) as $l) {
		if (strpos($l[0], '</') !== false) {
			$level++;
			if ($level == $below_level[$ol_level]) {
				$inside[$ol_level][0]['start'] = $l[1] + strlen($l[0]);
			}
			if (strpos($l[0], '</ol>') !== false && $ol_level < $max_ol_level -1 && ! isset($entered[$ol_level +1])) {
				$ol_level++;
				$below_level[$ol_level] = $level +1;
				$inside[$ol_level][0]['end'] = $l[1];
				$entered[$ol_level] = 1;
			}
		} elseif ($level <= 0) {
			$open[0] = $l[0];
			$inside[$ol_level][0]['start'] = $l[1] + strlen($l[0]);
			break;
		} else {
			if (strpos($l[0], '<ol') !== false && $level < $below_level[$ol_level] && @$entered[$ol_level] == 1) {
				$open[$ol_level] = $l[0];
				$inside[$ol_level][0]['start'] = $l[1] + strlen($l[0]);
				$entered[$ol_level] = 2;
				$ol_level--;
			}
			if ($level == $below_level[$ol_level]) {
				array_unshift($inside[$ol_level], array('end' => $l[1]));
			}
			$level--;
		}
	}
	for ($ol_level = 0 ; $ol_level < $max_ol_level ; $ol_level++) {
		if (preg_match('/start=[\'"](\d+)[\'"]/', $open[$ol_level], $start)) {
			$start_num = intval(@$start[1]);
		} else {
			$start_num = 1;
		}
		$inside_html = '';
		foreach ($inside[$ol_level] as $offset) {
			$inside_html .= substr($balanced, $offset['start'], $offset['end'] - $offset['start']);
		}
		$start_num += preg_match_all('/<li>/', $inside_html, $items);
		$ol_pos = strpos($start_tags, '<ol>'); // should be matched
		if (strpos($start_tags, '<li>', $ol_pos) == $ol_pos + 4) {
			$start_num -= 1; // use same number for splited item
		}
		if ($start_num > 1) {
			$start_html = ' start="' . $start_num . '"';
		} else {
			$start_html = ' start="1"';
		}
		preg_match('/\stype=[\'"][^\'"]+[\'"]/', $open[$ol_level], $type);
		$start_tags = preg_replace('/<ol>/', '<ol' . $start_html . @$type[0] . '>', $start_tags, 1);
	}
	return str_replace(' start="1"', '', $start_tags);
}

/* ==================================================
 * @param	string  $content
 * @return	string  $content
 */
public function trim_images($content) {
	if ($this->base->get('cache_size') > 0) {
		$total_size = strlen($content);
		for ($offset = 0, $replace = 'X'; 
		     preg_match('!<img ([^>]*?)src=([\'"])([^\\\\>]*?(\\\\.[^\\2\\\\>]*?)*)\\2([^>]*?) ?/?>!s', $content, $i, PREG_OFFSET_CAPTURE, $offset); //"syntax highlighting fix
		     $offset += strlen($replace)) {
			$offset  = $i[0][1];
			$replace = $i[0][0];
			$src     = $i[3][0];
			$attr    = $i[1][0] . $i[5][0];
			if (preg_match($this->wpurl_regex, $src)) {
				$imagesize = @filesize(preg_replace($this->wpurl_regex, $this->basepath, $src));
				if ($imagesize) {
					$total_size += $imagesize;
				}
				if ($total_size > $this->base->get('cache_size')) {
					if (preg_match('/alt=([\'"])([^\\\\>]*?(\\\\.[^\\1\\\\>]*?)*)\\1/s', $attr, $a)) { //"syntax highlighting fix
						$replace = $a[2];
					} else {
						$replace = basename($src);
					}
					$content = substr_replace($content, $replace, $offset, strlen($i[0][0]));
				}
			}
		}
	}
	return $content;
}

/* ==================================================
 * @param	string  $output
 * @return	string  $output
 */
public function shrink_dropdown($output) {
	$output = preg_replace(
		array('/ class=([\'"])[-_ \w]+\1/', '/[\r\n\t]/'),
		array('', ''),
		$output);
	$output = str_replace('&nbsp;', '-', $output);
	return $output;
}

/* ==================================================
 * @param	string  $content
 * @param	array   $tags
 * @param	array   $args
 * @return	string  $content
 */
public function shrink_tag_cloud($content, $tags, $args) {
	for ($offset = 0, $replace = 'X' ; 
	     preg_match('!<a href=([\'"])([^\\\\]*?(\\\\.[^\\1\\\\]*?)*)\\1([^>]*?)>(.*?)</a>!s', $content, $l, PREG_OFFSET_CAPTURE, $offset); 
	     $offset += strlen($replace))
	{
		$orig    = $l[0][0];
		$offset  = $l[0][1];
		$q       = $l[1][0];
		$href    = $l[2][0];
		$attr    = $l[4][0];
		$label   = $l[5][0];
		$replace = $orig;
		preg_match('/ style=([\'"])[^\\\\]*?(\\\\.[^\\1\\\\]*?)*\\1/s', $attr, $style);
		$replace = '<a href=' . $q . $href . $q . $style[0] . '>' . $label . '</a>';
		$content = substr_replace($content, $replace, $offset, strlen($orig));
	}
	return $content;
}

/* ==================================================
 * @param	string  $buffer
 * @return	string  $buffer
 */
public function add_url_inline_image($buffer) {
	$value = ks_is_image_inline() ? 'inline' : 'link';
	for ($offset = 0, $replace = 'X' ; 
	     preg_match('!<a ([^>]*?)href=([\'"])([^\\\\]*?(\\\\.[^\\2\\\\]*?)*)\\2([^>]*?)>!s', $buffer, $l, PREG_OFFSET_CAPTURE, $offset) ; 
	     $offset += strlen($replace))
	{
		$orig    = $l[0][0];
		$offset  = $l[0][1];
		$q       = $l[2][0];
		$href    = $l[3][0];
		$href    = _ks_quoted_remove_query_arg('img', $href);
		$attr1   = $l[1][0];
		$attr2   = $l[5][0];
		$replace = $orig;
		if (! preg_match($this->url_regex, $href) || preg_match($this->wpcontent_regex, $href) || preg_match('/id="inline"/', $attr1 . $attr2)) {
			continue;
		}
		$href .= (strpos($href, '?') === false ? '?' : '&amp;' ) . "img=$value";
		$replace = "<a {$attr1}href=$q$href$q$attr2>"; 
		$buffer = substr_replace($buffer, $replace, $offset, strlen($orig)); // convert links		
	}
	return $buffer;
}

/* ==================================================
 * @param	string  $link
 * @param	array   $args
 * @param	object  $comment
 * @param	object  $post
 * @return	none
 */
function fix_comment_reply_link($link, $args, $comment, $post) {
	if (strpos($link, 'wp-login.php?redirect_to=')) {
		$link = '<a href="' . ks_plugin_url(KTAI_NOT_ECHO) . 'login.php?redirect_to=' . urlencode(add_query_arg('replytocom', $comment->comment_ID, ks_comments_post_url($post->ID))) . '">' . $args['login_text'] . '</a>';
	} else {
		$co_post = add_query_arg('replytocom', $comment->comment_ID, ks_comments_post_url($post->ID));
		if ($post->post_password && ! ks_post_password_required($post)) {
			if (preg_match('!<img localsrc="\w+"[^>]*?>!s', $args['reply_text'], $icon)) {
				$icon = $icon[0];
				$reply_text = strip_tags($args['reply_text']);
			} else {
				$icon = '';
				$reply_text = strip_tags($args['reply_text']);
			}
			$link = _ks_internal_link($co_post, '', $reply_text, $post->post_password, $args['before'] . $icon, $args['after']);
		} else {
			$link = $args['before'] . '<a href="' . attribute_escape($co_post) . '">' . $args['reply_text'] . '</a>' . $args['after'];

		}
	}
	return $link;
}

/* ==================================================
 * @param	string  $result
 * @return	string  $result
 */
function fix_comments_pagenum_link($result) {
	$result = preg_replace('/#comments$/', '', $result);
	if (! preg_match('/(comment-page-\d+|cpage=\d+)/', $result)) {
		global $post;
		$result = ks_get_comments_list_link($post->ID);
	}
	return $result;
}

/* ==================================================
 * @param	string  $null
 * @param	mix     $attr
 * @param	string  $content
 * @return	string  $content
 * based on img_caption_shortcode() at wp-includes/media.php of WP 2.8.4
 */
function img_caption($null, $attr, $content = null) {
	extract(shortcode_atts(array(
		'id'	=> '',
		'align'	=> 'alignnone',
		'width'	=> '',
		'caption' => '',
		'margin' => 2,
	), $attr));
	if ( 1 > (int) $width || empty($caption) ) {
		return $content;
	}
	return ks_image_alignment(do_shortcode($content), $align, $margin);
}

// ===== End of class ====================
}

global $Ktai_Style, $KS_Shrinkage;
$Ktai_Style->shrinkage = $KS_Shrinkage = new KtaiShrinkage;

if (! class_exists('KtaiError')) :
/* ==================================================
 *   KtaiError class
   ================================================== */

function is_ks_error($thing) {
	return (is_object($thing) && is_a($thing, 'KtaiError'));
}

class KtaiError extends Exception {

public function setCode($code) {
	$this->code = $code;
}

// ===== End of class ====================
}
endif;
?>